float4x4 matWorld;
float4x4 matWorldViewProj;
float4 vecViewPos;
float4 vecLightColor[8];
float4 vecLightPos[8];

float3x3 matTangent;

texture entSkin1;
texture entSkin2;

sampler2D smpDiffuse = sampler_state
{
	Texture = <entSkin1>;
	
	MinFilter	= linear;
	MagFilter	= linear;
	MipFilter	= linear;
	
	AddressU		= Wrap;
	AddressV		= Wrap;
};

sampler2D smpNormal = sampler_state
{
	Texture = <entSkin2>;
	
	MinFilter	= linear;
	MagFilter	= linear;
	MipFilter	= linear;
	
	AddressU		= Wrap;
	AddressV		= Wrap;
};

struct VS_OUT
{
	float4 Pos : POSITION;
	float2 Tex : TEXCOORD0;
	float3 WorldPos : TEXCOORD1;
	float3 tangent0 : TEXCOORD2;
	float3 tangent1 : TEXCOORD3;
	float3 tangent2 : TEXCOORD4;
};

VS_OUT Shader_VS(float4 inPos : POSITION, float2 inTex : TEXCOORD0, float3 inTangent : TEXCOORD2, float3 inNormal : NORMAL)
{
	VS_OUT Out = (VS_OUT)0;
	
	inNormal = -inNormal;
	
	Out.tangent0 = mul(inTangent, matWorld);
	Out.tangent1 = mul(cross(inTangent, inNormal), matWorld);
	Out.tangent2 = mul(inNormal, matWorld);
	
	Out.Pos = mul(inPos, matWorldViewProj);
	
	Out.Tex = inTex;
	Out.WorldPos = mul(inPos, matWorld);
	
	return Out;
}

float4 Shader_PS(float2 inTex : TEXCOORD0, float3 inPos : TEXCOORD1, float3 tangent0 : TEXCOORD2, float3 tangent1 : TEXCOORD3, float3 tangent2 : TEXCOORD4) : COLOR
{
	matTangent[0] = tangent0;
	matTangent[1] = tangent1;
	matTangent[2] = tangent2;
	
	float4 Color = tex2D(smpDiffuse, inTex);
	float4 SpecNormal = tex2D(smpNormal, inTex);
	float3 Normal = SpecNormal.rgb*2.0-1.0;
	Normal = mul(Normal, matTangent);
	Normal = normalize(Normal);
	
	float3 lightdir = inPos-vecLightPos[0].xyz;
	float att = saturate(1.0-length(lightdir)/vecLightPos[0].w);
	lightdir = normalize(lightdir);
	
	float ambient = 0.1;
	float diffuse = saturate(dot(lightdir, Normal));
	float3 viewdir = normalize(inPos-vecViewPos);
	float3 halfvec = normalize(viewdir+lightdir);
	float specularity = saturate(dot(halfvec, Normal));
	float shading = ambient+diffuse*vecLightColor[0]*att;
	
	Color *= shading;
	Color += vecLightColor[0]*pow(specularity, 10)*diffuse*SpecNormal.a;
	
	Color.a = 1.0;
	
	return Color;
}

technique tech_00
{
	pass pass_00
	{
		VertexShader = compile vs_2_0 Shader_VS();
		PixelShader = compile ps_2_0 Shader_PS();
	}
}
